use crate::utils::{
    request::{Request, Response},
    task_log,
};
use storage::services::{
    aws::{S3Client, S3ClientConfig},
    ObjectStorageClient,
};

pub fn register(pool: &sqlx::Pool<sqlx::Postgres>) {
    let p = pool.clone();
    tokio::spawn(async move {
        let _ = upload_sitemap(&p).await;
    });
}

async fn upload_sitemap(
    pool: &sqlx::Pool<sqlx::Postgres>,
) -> Result<(), Box<dyn std::error::Error>> {
    // start task logging
    task_log::start(3, pool).await?;

    // get request and request the things
    let request = Request::new();
    let sitemap_url = format!("{}/posts/sitemap", request.base_url);
    let sitemap_result = request.request_url::<String>(&sitemap_url).await.unwrap();

    // upload the sucker to obj storage
    if let Response::Xml(sitemap) = sitemap_result {
        let client_config = S3ClientConfig::from_env().unwrap();
        let s3_client = S3Client::new(&client_config);
        let _ = s3_client
            .put_object(
                client_config.bucket.as_str(),
                "sitemap.xml",
                sitemap.as_bytes().to_vec(),
            )
            .await;
        println!("Finished uploading sitemap!");
    }

    Ok(())
}
